<?php

namespace Drupal\search_api_default_content_deploy\EventSubscriber;

use Drupal\default_content_deploy\Event\DefaultContentDeployEvents;
use Drupal\default_content_deploy\Event\IndexAwareEventInterface;
use Drupal\default_content_deploy\Event\PostSerializeEvent;
use Drupal\search_api\Event\IndexingItemsEvent;
use Drupal\search_api\Event\SearchApiEvents;
use Drupal\search_api_default_content_deploy\Plugin\search_api\backend\DefaultContentDeployBackend;
use Symfony\Component\EventDispatcher\EventSubscriberInterface;

/**
 * Event subscriptions for events dispatched by default_content_deploy.
 */
class DefaultContentDeployEventSubscriber implements EventSubscriberInterface {

  protected string $indexId = '';

  /**
   * {@inheritdoc}
   */
  public static function getSubscribedEvents() {
    $events[DefaultContentDeployEvents::PRE_SERIALIZE] = [['setIndexId', 1000]];
    $events[DefaultContentDeployEvents::POST_SERIALIZE] = [['setIndexId', 1000], ['adjustLinkDomain', 1000]];
    $events[DefaultContentDeployEvents::PRE_SAVE] = [['setIndexId', 1000]];
    $events[SearchApiEvents::INDEXING_ITEMS] = [['identifyIndex']];

    return $events;
  }

  public function setIndexId(IndexAwareEventInterface $event): void {
    $event->setIndexId($this->indexId);
  }

  public function identifyIndex(IndexingItemsEvent $event): void {
    $server = $event->getIndex()->getServerInstance();
    if ($server->hasValidBackend() && $server->getBackend() instanceof DefaultContentDeployBackend) {
      $this->indexId = $event->getIndex()->id();
    }
  }

  public function adjustLinkDomain(PostSerializeEvent $event): void {
    if ($event->getIndexId()) {
      /** @var \Drupal\default_content_deploy\Exporter $exporter */
      $exporter = \Drupal::service('default_content_deploy.exporter');
      /** @var \Drupal\default_content_deploy\DeployManager $deploy_manager */
      $deploy_manager = \Drupal::service('default_content_deploy.manager');

      $link_domain = $exporter->getLinkDomain();
      $current_host = $deploy_manager->getCurrentHost();

      if ($link_domain !== $current_host) {
        $link_domain = str_replace('/', '\/', $link_domain);
        $current_host = str_replace('/', '\/', $current_host);
        $event->setContent(str_replace($current_host, $link_domain, $event->getContent()));
      }
    }
  }
}
