<?php

namespace Drupal\search_api_default_content_deploy\Plugin\search_api\backend;

use Drupal\Core\Entity\ContentEntityTypeInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\default_content_deploy\Exporter;
use Drupal\search_api\LoggerTrait;
use Drupal\Core\Config\Config;
use Drupal\Core\Entity\ContentEntityInterface;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Plugin\PluginDependencyTrait;
use Drupal\Core\Plugin\PluginFormInterface;
use Drupal\Core\StringTranslation\StringTranslationTrait;
use Drupal\search_api\Backend\BackendPluginBase;
use Drupal\search_api\IndexInterface;
use Drupal\search_api\Plugin\PluginFormTrait;
use Drupal\search_api\Plugin\search_api\datasource\ContentEntity;
use Drupal\search_api\Query\QueryInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Default Content Deploy backend for search api.
 *
 * @SearchApiBackend(
 *   id = "search_api_default_content_deploy",
 *   label = @Translation("Default Content Deploy"),
 *   description = @Translation("'Leverage the Search API infrastructure to track and incrementally export content..")
 * )
 */
class DefaultContentDeployBackend extends BackendPluginBase implements PluginFormInterface {

  use PluginFormTrait {
    PluginFormTrait::submitConfigurationForm as traitSubmitConfigurationForm;
  }

  use PluginDependencyTrait;

  use StringTranslationTrait;

  use LoggerTrait;

  /**
   * The exporter.
   *
   * @var \Drupal\default_content_deploy\Exporter
   */
  protected $exporter;

  /**
   * The entity type manager.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * {@inheritdoc}
   */
  public function __construct(array $configuration, $plugin_id, array $plugin_definition, Exporter $exporter, EntityTypeManagerInterface $entityTypeManager) {
    parent::__construct($configuration, $plugin_id, $plugin_definition);

    $this->exporter = $exporter;
    $this->entityTypeManager = $entityTypeManager;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $container->get('default_content_deploy.exporter'),
      $container->get('entity_type.manager'),
    );
  }

  /**
   * {@inheritdoc}
   *
   * @throws \Drupal\Component\Plugin\Exception\PluginException
   * @throws \Drupal\search_api\SearchApiException
   */
  public function buildConfigurationForm(array $form, FormStateInterface $form_state) {
    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function getDiscouragedProcessors() {
    return [
      'content_access',
      'double_quote_workaround',
      'highlight',
      'html_filter',
      'ignorecase',
      'ignore_character',
      'number_field_boost',
      'snowball_stemmer',
      'solr_boost_more_recent',
      'solr_regex_replace',
      'solr_dummy_fields',
      'stemmer',
      'stopwords',
      'tokenizer',
      'transliteration',
      'type_boost',
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function removeIndex($index) {
    parent::removeIndex($index);

    // @todo delete directories?
  }

  /**
   * {@inheritdoc}
   *
   * @throws \Drupal\Component\Plugin\Exception\PluginException
   * @throws \Drupal\search_api\SearchApiException
   */
  public function indexItems(IndexInterface $index, array $items) {
    $ret = [];

    $index_third_party_settings = $index->getThirdPartySettings('search_api_default_content_deploy') + search_api_default_content_deploy_default_index_third_party_settings();

    $this->exporter->setFolder($index_third_party_settings['content_directory']);
    //$this->exporter->setSkipComputedFields($index_third_party_settings['skip_computed_fields']);
    $this->exporter->setTextDependencies($index_third_party_settings['text_dependencies']);
    $this->exporter->setSkipEntityTypeIds($index_third_party_settings['skip_entity_types'] ?? []);
    $this->exporter->setLinkDomain($index_third_party_settings['link_domain']);

    foreach ($items as $item) {
      $datasource = $item->getDatasource();
      if ($datasource instanceof ContentEntity) {
        /** @var ContentEntityInterface $entity */
        $entity = $item->getOriginalObject()->getEntity();
        if (!$this->exporter->exportEntity($entity, $index_third_party_settings['export_referenced_entities'])) {
          // @todo log skipped entity?
        }
        $ret[] = $item->getId();
      }
    }

    return $ret;
  }

  /**
   * {@inheritdoc}
   */
  public function deleteItems(IndexInterface $index, array $ids) {
    // @todo
  }

  /**
   * {@inheritdoc}
   */
  public function deleteAllIndexItems(IndexInterface $index, $datasource_id = NULL) {
    // @todo
  }

  /**
   * {@inheritdoc}
   */
  public function search(QueryInterface $query) {
  }
}
