<?php

namespace Drupal\search_api_default_content_deploy\Plugin\search_api\processor;

use Drupal\Core\TypedData\TranslatableInterface;
use Drupal\search_api\Plugin\search_api\processor\EntityStatus;

/**
 * Excludes non-default translations.
 *
 * @SearchApiProcessor(
 *   id = "dcd_default_translation",
 *   label = @Translation("Default translation"),
 *   description = @Translation("Exclude non-default translations."),
 *   stages = {
 *     "alter_items" = 0,
 *   },
 * )
 */
class DefaultTranslation extends EntityStatus {

  /**
   * {@inheritdoc}
   */
  public function alterIndexedItems(array &$items) {
    /** @var \Drupal\search_api\Item\ItemInterface $item */
    foreach ($items as $item_id => $item) {
      $object = $item->getOriginalObject()->getValue();
      $default_translation = TRUE;
      if ($object instanceof TranslatableInterface) {
        $default_translation = $object->isDefaultTranslation();
      }

      if (!$default_translation) {
        unset($items[$item_id]);
      }
    }
  }

}
